// BanditSpiel.java                             MM 2010

/**
 * Der einarmige Bandit, Spiel fuer eine Person.
 *
 * Anfangkapital des Spielers: 300 Euro
 * Anfangkapital des Bandit:   200 Euro
 *
 * Drei zufaellige Zahlen 0..9 werden gewuerfelt.
 * Sind alle drei Zahlen gleich, wird das Vierfache,
 * sind zwei Zahlen gleich, das Doppelte
 * des Einsatzes zurueckgezahlt.
 * Sonst wird der Einsatz vom Kapital abgezogen.
 *
 * Das Spiel endet,
 * falls der Spieler oder der Bandit Pleite ist.
 */
public class BanditSpiel
{
/* ------------------------------------------------- */
                                           // coreData
/**
 * Spielautomat.
 */
  private Bandit bandit;

/**
 * Spieler.
 */
  private Konto spieler;

/**
 * Einsatz.
 */
  private int einsatz;

/* ------------------------------------------------- */
                                         // Konstruktor
/**
 * Konstruktor, setzt Standardwerte.
 */
  public BanditSpiel()
  {
    reset();
  }

/* ------------------------------------------------- */
                                    // toString-Methode
/**
 * Darstellen des Automatenzustandes mit aktuellem
 * Kapital und aktueller Zahlenanzeige als String.
 * @return Spielzustand
 */
  public String toString()
  {
    String str = "\nBandit:  " + bandit;      // Bandit
    str += "\nSpieler: " + spieler;          // Spieler

    return str;
  }

/* ------------------------------------------------- */
                                         // Spielregeln
/**
 * Erlaeuterung des Spiels.
 * @return Spielregeln
 */
  public String spielRegeln()
  {
    String str = "Der einarmige Bandit, ";
    str += "Spiel fuer eine Person.\n\n";

    str += "Anfangkapital des Spielers: 300 Euro\n";
    str += "Anfangkapital des Bandit: 200 Euro\n\n";

    str += "Drei zufaellige Zahlen 0..9 werden ";
    str += "gewuerfelt.\n";
    str += "Sind alle drei Zahlen gleich, ";
    str += "wird das Vierfache,\n";
    str += "sind zwei Zahlen gleich, das Doppelte ";
    str += "des Einsatzes zurueckgezahlt.\n";
    str += "Sonst wird der Einsatz vom Kapital ";
    str += "abgezogen.\n\n";

    str += "Das Spiel endet, ";
    str += "falls der Spieler oder ";
    str += "der Bandit Pleite ist.";

    return str;
  }

/* ------------------------------------------------- */
                                     // Spielauswertung
/**
 * Auswertung eines Spiels.
 * @return Verlierer als String
 */
  public String auswertung()
  {
    String str = "";
    if( spieler.istPleite())
      str = "Du bist Pleite!";
    if( bandit.istPleite())
      str = "Der Bandit ist Pleite!";

    return str;
  }

/* ------------------------------------------------- */
                                        // get-Methoden
/**
 * Liest aktuelles Haben des Bandit.
 * @return Bandithaben
 */
  public int getHabenBandit()
  {
    return bandit.getHaben();
  }

/**
 * Liest aktuelles Haben des Spielers.
 * @return Spielerhaben
 */
  public int getHabenSpieler()
  {
    return spieler.getHaben();
  }

/**
 * Liest aktuelle Zahlenanzeige des Automaten.
 * @return aktuelle Zahlen
 */
  public int[] getZahlen()
  {
    return bandit.getZahlen();
  }

/* ------------------------------------------------- */
                                    // service-Methoden
/**
 * Setzt und verbucht Einsatz auf Spielerkonte:
 * 0 < Einsatz <= Spielerhaben.
 * @param euro gewuenschtes Einsatz
 * @return true, Einsatz wurde gesetzt
 */
  public boolean setEinsatz( int euro)
  {
    einsatz = 0;
    if( euro > 0 && spieler.verbuchen( -euro))
      einsatz = euro;

    return einsatz != 0;
  }

/**
 * Spielzug, Ziehen des Automatenarmes,
 * verbucht Gewinn auf Spielerkonto.
 * @return true, Spiel noch nicht zu Ende
 */
  public boolean ziehen()
  {
    if( einsatz != 0) 
      spieler.verbuchen( bandit.ziehen( einsatz));
    einsatz = 0;

    return !spieler.istPleite() && !bandit.istPleite();
  }

/**
 * Setzt Standardwerte zurueck.
 */
  public void reset()
  {
    bandit = new Bandit( 200);
    spieler = new Konto( 300);
    einsatz = 0;
  }
}
