// BanditSpielController.java                   MM 2010
// MVC-Controller

import java.awt.event.*; // WindowAdapter, Listener,...
import javax.swing.*;                    // JOptionPane

/**
 * Controller zum BanditSpielView.
 */
public class BanditSpielController
  extends WindowAdapter
  implements ActionListener, KeyListener
{
/**
 * Mathematisches Model,
 * enthaelt Funktionalitaet der Zahlenspiegelung.
 */
  private BanditSpielModel model;

/**
 * Zum Controller gehoeriger View.
 */
  private BanditSpielView view;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor, initialisiert Model und View.
 * @param view zum Controller gehoeriger View
 */
  public BanditSpielController
  ( BanditSpielModel model, BanditSpielView view)
  {
    this.model = model;
    this.view = view;
  }

/* ------------------------------------------------- */
                 // handleEvent-Methoden ActionListener
/**
 * ActionListener,
 * Ereignisauswertung fuer Buttons.
 */
  public void actionPerformed( ActionEvent ae)
  {
    Object command = ae.getActionCommand();

// Fuehrt Spielzug aus
    if( command.equals( view.ACTION_ZUG))
    {
      if( model.ziehen()) view.setTfEinsatz();
      else view.setBtNeu();         // einer ist Pleite
    }

// Ruft Spielregeln auf
    if( command.equals( view.ACTION_REGEL))
    {
      setDialog( model.spielRegeln());
      view.setTfEinsatz();             // neuer Einsatz
    }

// Setzt Startzustand
    if( command.equals( view.ACTION_NEU))
    {
      model.reset();             // setzt Model zurueck
      view.reset();               // setzt View zurueck
    }

// Beendet Programm
    if( command.equals( view.ACTION_ENDE))
      view.release();
 }

/* ------------------------------------------------- */
                    // handleEvent-Methoden KeyListener
/**
 * KeyListener,
 * Ereignisauswertung Taste wurde gedrueckt,
 * Methode tut nichts.
 */
  public void keyPressed( KeyEvent ke)
  {
  }

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde losgelassen.
 */
  public void keyReleased( KeyEvent ke)
  {
                         // schliesst Einsatzeingabe ab
    if( view.istTfEinsatz( ke.getSource())
        && ke.getKeyChar() == '\n')
    {
      try
      {
        int einsatz
        = Integer.valueOf( view.getTfEinsatz());

        if( model.setEinsatz( einsatz))
          view.setBtZug();
        else 
        {
          if( einsatz == 0)
            setDialog( "Mindestens einen Euro eingeben!");
          else
            setDialog( "So viel Euro hast Du nicht!");
          view.setTfEinsatz();         // neuer Einsatz
        }
      }
      catch( Exception e)
      {
        setDialog( "Nur Zahlen eingeben!");
        view.setTfEinsatz();           // neuer Einsatz
      }
    }
  }

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde gedrueckt.
 * und losgelassen, Methode tut nichts.
 */
  public void keyTyped( KeyEvent ke)
  {
  }

/* ------------------------------------------------- */
                  // handleEvent-Methoden WindowAdapter
/**
 * WindowAdapter,
 * Schliessen des Models.
 */
  public void windowClosing( WindowEvent we)
  {
      view.release();
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Freigabe der Controllers,
 * setzt Model und View zurck.
 */
  public void release()
  {
    view = null;
    model = null;
  }

/* ------------------------------------------------- */

/**
 * Eingabefehlernmitteilung, Ausgabe der Spielregeln.
 * @param text Meldung
 */
  private void setDialog( String text)
  {
    JOptionPane.showMessageDialog( view, text,
    "Meldung", JOptionPane.ERROR_MESSAGE);
  }
}