// BanditSpielView.java                         MM 2010
// MVC-View

import java.awt.*;                             // Color
import javax.swing.*;                // GUI-Komponenten
import java.util.*;                         // Observer

/**
 * View zum BanditSpielModel.
 */
public class BanditSpielView
  extends JFrame
  implements Observer
{
/* ------------------------------------------------- */
                                      // GUI-Konstanten
/**
 * Konstante, Spiel starten.
 */
  public final String ACTION_ZUG = "Zug";

/**
 * Konstante, Spiel wiederholen.
 */
  public final String ACTION_REGEL = "Regeln";

/**
 * Konstante, Spiel wiederholen.
 */
  public final String ACTION_NEU = "Neu";

/**
 * Konstante, Programm beenden.
 */
  public final String ACTION_ENDE = "Beenden";

/* ------------------------------------------------- */
                                     // GUI-Komponenten
/**
 * Button Regeln.
 */
  private JButton btRegel;

/**
 * Button Zug.
 */
  private JButton btZug;

/**
 * Button Neu.
 */
  private JButton btNeu;

/**
 * Button Beenden.
 */
  private JButton btBeenden;

/**
 * Textfeld zur Eingabe des Einsatzes.
 */
  private JTextField tfEinsatz;

/**
 * Label Fertig.
 */
  private JLabel lbFertig;

/**
 * Label Haben des Spielers.
 */
  private JLabel lbHabenSpieler;

/**
 * Label Haben des Spielautomaten.
 */
  private JLabel lbHabenBandit;

/**
 * Label Zahl 1.
 */
  private JLabel lbZahl1;

/**
 * Label Zahl 2.
 */
  private JLabel lbZahl2;

/**
 * Label Zahl 3.
 */
  private JLabel lbZahl3;

/* ------------------------------------------------- */
                                                 // MVC
/**
 * Mathematisches Model,
 * enthaelt Funktionalitaet der Zahlenspiegelung.
 */
  private BanditSpielModel model;

/**
 * Zum View gehoeriger Cotroller.
 */
  private BanditSpielController controller;

/* ------------------------------------------------- */
                                    // MVC-Installation
/**
 * Konstruktor, setzt Ueberschrift und installiert MVC.
 * @param model Model, welches dargestellt werden soll
 * @param titel Ueberschrift
 */
  public BanditSpielView
  ( BanditSpielModel model, String titel)
  {
// Titelzeile
    super( titel);
// Model
    this.model = model;
    this.model.addObserver( this);
// Controller
    controller = makeController();
// View
    makeView();
  }

/**
 * Erzeugt Controller, Empfaenger fuer Ereignisse.
 * @return Controller fuer View
 */
  private BanditSpielController makeController()
  {
    return new BanditSpielController( model, this);
  }

/**
 * Erzeugt View,
 * baut die grafische Oberflaeche auf,
 * Controller verarbeitet Fensterereignisse.
 */
  private void makeView()
  {
// ContentPane
    getContentPane().add( createBanditSpielBox());

// Startzustand
    reset();

// Fenster
    addWindowListener( controller);
    pack();
    setVisible( true);
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                         // ContentPane
/**
 * Senkrechte Box fuer Ein-, Ausgaben und Fertigstatus.
 */
  private Box createBanditSpielBox()
  {                                      // Fertiglabel
//    lbFertig = new JLabel( ACTION_FERTIG);
    lbFertig = new JLabel( model.auswertung());
    lbFertig.setForeground( Color.RED);
    lbFertig.setAlignmentX( Component.CENTER_ALIGNMENT);

    Box box = Box.createVerticalBox();

    box.add( Box.createVerticalStrut(20));
    box.add( createBoxInOut());    // Ein- und Ausgaben
    box.add( Box.createVerticalStrut(20));
    box.add( createBoxBt());            // Steuerbutten
    box.add( Box.createVerticalStrut(20));
    box.add( lbFertig);                // Fertigmeldung
    box.add( Box.createVerticalStrut(20));

    return box;
  }

/**
 * Waagerechte Box fuer Ein- und Ausgaben.
 */
  private Box createBoxInOut()
  {
    Box box = Box.createHorizontalBox();

    box.add( Box.createHorizontalStrut(20));
    box.add( createBoxBandit());           // Banditbox
    box.add( Box.createHorizontalStrut(50));
    box.add( createBoxSpieler());         // Spielerbox
    box.add( Box.createHorizontalStrut(20));

    return box;
  }

/**
 * Senkrechte Box fuer Bandit.
 */
  private Box createBoxBandit()
  {
    JLabel lbBandit = new JLabel( "Bandit");
    lbBandit.setAlignmentX( Component.LEFT_ALIGNMENT);

    Box box1 = createBoxZahlen();     // Ausgabe Zahlen
    box1.setAlignmentX( Component.LEFT_ALIGNMENT);

    Box box2 = createBoxHabenBandit(); // Ausgabe Haben
    box2.setAlignmentX( Component.LEFT_ALIGNMENT);

    Box box = Box.createVerticalBox();
    box.add( lbBandit);
    box.add( Box.createVerticalStrut(20));
    box.add( box1);
    box.add( Box.createVerticalStrut(20));
    box.add( box2);

    return box;
  }

/**
 * Senkrechte Box fuer Spieler.
 */
  private Box createBoxSpieler()
  {
    JLabel lbSpieler = new JLabel( "Spieler");
    lbSpieler.setAlignmentX( Component.RIGHT_ALIGNMENT);

    Box box1 = createBoxEinsatz();   // Eingabe Einsatz
    box1.setAlignmentX( Component.RIGHT_ALIGNMENT);

    Box box2 = createBoxHabenSpieler();// Ausgabe Haben
    box2.setAlignmentX( Component.RIGHT_ALIGNMENT);

    Box box = Box.createVerticalBox();
    box.add( lbSpieler);
    box.add( Box.createVerticalStrut(18));
    box.add( box1);
    box.add( Box.createVerticalStrut(18));
    box.add( box2);

    return box;
  }

/**
 * Waagerechte Box fuer Zahleneingabe,
 * Controller verarbeitet Eingaben.
 */
  private Box createBoxZahlen()
  {
    lbZahl1 = new JLabel( "0");
    lbZahl2 = new JLabel( "0");
    lbZahl3 = new JLabel( "0");

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Zahlen:"));
    box.add( Box.createHorizontalStrut(20));
    box.add( lbZahl1);
    box.add( Box.createHorizontalStrut(20));
    box.add( lbZahl2);
    box.add( Box.createHorizontalStrut(20));
    box.add( lbZahl3);

    return box;
  }

/**
 * Waagerechte Box fuer Zahleneingabe,
 * Controller verarbeitet Eingaben.
 */
  private Box createBoxHabenBandit()
  {
    lbHabenBandit = new JLabel
    ( String.valueOf( model.getHabenBandit()));

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Haben:"));
    box.add( Box.createHorizontalStrut(20));
    box.add( lbHabenBandit);

    return box;
  }

/**
 * Waagerechte Box fuer Zahleneingabe,
 * Controller verarbeitet Eingaben.
 */
  private Box createBoxEinsatz()
  {                                      // Eingabefeld
    tfEinsatz = new JTextField( 3);
    tfEinsatz.setHorizontalAlignment( JTextField.RIGHT);
    tfEinsatz.addKeyListener( controller);

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Dein Einsatz:"));
    box.add( Box.createHorizontalStrut(20));
    box.add( tfEinsatz);

    return box;
  }

/**
 * Waagerechte Box fuer Zahleneingabe,
 * Controller verarbeitet Eingaben.
 */
  private Box createBoxHabenSpieler()
  {
    lbHabenSpieler = new JLabel
    ( String.valueOf( model.getHabenSpieler()));

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Haben:"));
    box.add( Box.createHorizontalStrut(20));
    box.add( lbHabenSpieler);

    return box;
  }

/**
 * Waagerechte Box fuer Steuerbutton,
 * Controller verarbeitet Buttonereignisse.
 */
  private Box createBoxBt()
  {
    btZug   = new JButton( ACTION_ZUG);
    btZug.addActionListener( controller);

    btNeu     = new JButton( ACTION_NEU);
    btNeu.addActionListener( controller);

    btRegel   = new JButton( ACTION_REGEL);
    btRegel.addActionListener( controller);

    btBeenden = new JButton( ACTION_ENDE);
    btBeenden.addActionListener( controller);

    Box box = Box.createHorizontalBox();

    box.add( btZug);
    box.add( btNeu);
    box.add( btRegel);
    box.add( btBeenden);

    return box;
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                    // service-Methoden fuer Controller
/**
 * Setzt Startzustand.
 */
  public void reset()
  {
    setTfEinsatz();
  }

/**
 * Ereignisquelle ist Textfeld.
 * @return true, falls Zahl eingegeben wurde
 */
  public boolean istTfEinsatz( Object quelle)
  {
    return quelle == tfEinsatz;
  }

/**
 * Liest Inhalt des Textfeld.
 * @return Einsatz als String
 */
  public String getTfEinsatz()
  {
    return tfEinsatz.getText();
  }

/**
 * Bereitet neue Einsatzeingabe vor.
 */
  public void setTfEinsatz()
  {
    tfEinsatz.setText( "");
    tfEinsatz.setEditable( true);
    btZug.setEnabled( false);

    tfEinsatz.requestFocus();
  }

/**
 * Einsatzeingabe abgeschlossen, Zug moeglich.
 */
  public void setBtZug()
  {
    tfEinsatz.setEditable( false);
    btZug.setEnabled( true);
    btZug.setText( ACTION_ZUG);

    btZug.requestFocus();
  }

/**
 * Spiel abgeschlossen, neues Spiel moeglich.
 */
  public void setBtNeu()
  {
    tfEinsatz.setEditable( false);
    btZug.setEnabled( false);

    btNeu.requestFocus();
  }

/* ------------------------------------------------- */
/* ------------------------------------------------- */
                                    // Observer-Methode
/**
 * Ueberschreiben der Interfacemethode update,
 * legt Reaktion auf Aenderungen fest.
 * @param m Model, welches Aenderungen meldet
 * @param o geanderte Objekte
 */
  public void update( Observable m, Object o)
  {
    if( m == model)
    {
      int[] zahlen = model.getZahlen();

      lbZahl1.setText( String.valueOf( zahlen[ 0]));
      lbZahl2.setText( String.valueOf( zahlen[ 1]));
      lbZahl3.setText( String.valueOf( zahlen[ 2]));

      lbHabenBandit.setText
      ( String.valueOf( model.getHabenBandit()));
      lbHabenSpieler.setText
      ( String.valueOf( model.getHabenSpieler()));

      lbFertig.setText( model.auswertung());
    }
  }

/* ------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Setzt Model und Controller zurck.
 */
  public void release()
  {
// Fenster
    dispose();
// Controller
    controller.release();
    controller = null;
// Model
    model.deleteObserver( this);
    model = null;
  }
}